from typing import List, Tuple

from i8_terminal.types.auto_complete_choice import AutoCompleteChoice


def get_price_periods() -> List[Tuple[str, str]]:
    return [
        ("1M", "One Month"),
        ("3M", "Three Months"),
        ("6M", "Six Months"),
        ("1Y", "One Year"),
        ("3Y", "Three Years"),
        ("5Y", "Five Years"),
    ]


class PricePeriodParamType(AutoCompleteChoice):
    name = "priceperiod"

    def get_suggestions(self, keyword: str, pre_populate: bool = True) -> List[Tuple[str, str]]:
        if not self.is_loaded:
            self.set_choices(get_price_periods())

        if pre_populate and keyword.strip() == "":
            return self._choices[: self.size]
        return self.search_keyword(keyword)

    def __repr__(self) -> str:
        return "PRICEPERIOD"



from typing import Any, Dict, Optional

import click
import investor8_sdk
import numpy as np
from rich.console import Console

from i8_terminal.commands.financials import financials
from i8_terminal.common.cli import pass_command
from i8_terminal.common.financials import (
    fin_df2export_df,
    fin_df2Tree,
    find_similar_statement,
    get_statements_codes,
    get_statements_disp_name,
    parse_identifier,
    prepare_financials_df,
)
from i8_terminal.common.metrics import get_all_financial_metrics_df
from i8_terminal.common.utils import export_data
from i8_terminal.config import APP_SETTINGS
from i8_terminal.types.fin_identifier_param_type import FinancialsIdentifierParamType
from i8_terminal.types.fin_statement_param_type import FinancialStatementParamType
from i8_terminal.types.period_type_param_type import PeriodTypeParamType


def get_standardized_financials(
    identifiers_dict: Dict[str, str],
    statement: str,
    period_type: Optional[str],
    period_size: int = 4,
    exportize: Optional[bool] = False,
) -> Optional[Dict[str, Any]]:
    fins = []
    if identifiers_dict.get("fiscal_period"):
        fins = [
            investor8_sdk.FinancialsApi().get_financials_single(
                ticker=identifiers_dict["ticker"],
                stat_code=statement,
                fiscal_year=identifiers_dict.get("fiscal_year"),
                fiscal_period=identifiers_dict.get("fiscal_period"),
            )
        ]
    else:
        period_type = "FY" if not period_type else period_type
        fins = investor8_sdk.FinancialsApi().get_list_standardized_financials(
            ticker=identifiers_dict["ticker"],
            stat_code=statement,
            period_type=period_type,
            end_year=identifiers_dict.get("fiscal_year", ""),
        )
    if not fins:
        return None
    return prepare_financials_df(fins, period_size, include_ticker=False, exportize=exportize)


@financials.command()
@click.option(
    "--identifier",
    "-i",
    type=FinancialsIdentifierParamType(),
    required=True,
    help="Comma-separated list of identifiers.",
)
@click.option(
    "--statement",
    "-s",
    type=FinancialStatementParamType(),
    default="income",
    help="Type of financial statement.",
)
@click.option(
    "--period_type",
    "-m",
    type=PeriodTypeParamType(),
    help="Period by which you want to view the report. Possible values are `FY` for yearly, `Q` for quarterly, and `TTM` for TTM reports.",
)
@click.option("--export", "export_path", "-e", help="Filename to export the output to.")
@pass_command
def list(identifier: str, statement: str, period_type: Optional[str], export_path: Optional[str]) -> None:
    """
    Lists financial metrics of a given company.

    Examples:

    `i8 financials list --period_type FY --statement income --identifier AAPL-2020-FY`

    """
    matched_statement = find_similar_statement(statement)
    if not matched_statement:
        click.echo(
            f"`{statement}` is not a valid statement code. \nValid statement codes: {', '.join(get_statements_codes())}"
        )
        click.echo("Unknown Statement Code!")
        return
    identifiers_dict = parse_identifier(identifier, period_type)
    console = Console()
    with console.status("Fetching data...", spinner="material") as status:
        fins = get_standardized_financials(
            identifiers_dict, matched_statement, period_type, period_size=4, exportize=True if export_path else False
        )
        if fins is None:
            status.stop()
            click.echo("No data found!")
            return
        periods_list = fins["data"].columns[1:].to_list()
        df_metrics = get_all_financial_metrics_df()
        df = fins["data"].merge(df_metrics, on="tag", how="left")
        df["section_name"] = df["section_name"].apply(lambda x: "Others" if not x or x == "-" else x)
        df = df.astype(object).replace(np.nan, None)  # Replace nan with None

    if export_path:
        export_df = fin_df2export_df(df, periods_list)
        export_data(
            export_df,
            export_path,
            column_width=18,
            column_format=APP_SETTINGS["styles"]["xlsx"]["financials"]["column"],
        )
    else:
        tree = fin_df2Tree(
            df,
            fins["header"],
            periods_list,
            title=f"{identifiers_dict['ticker'].upper()} {get_statements_disp_name(matched_statement)}",
        )
        console.print(tree)



from typing import List, Tuple

from i8_terminal.types.auto_complete_choice import AutoCompleteChoice


def get_fin_periods() -> List[Tuple[str, str]]:
    all_periods = [("FY", "Anual")]
    for i in range(1, 5):
        all_periods.append((f"Q{i}", f"Fiscal Quarter {i}"))
        all_periods.append((f"Q{i}TTM", f"Trailing 12 Months Ending Q{i}"))
        all_periods.append((f"Q{i}YTD", f"Year to Date Ending Q{i}"))
    return all_periods


class FinancialsPeriodParamType(AutoCompleteChoice):
    name = "financialsperiod"

    def get_suggestions(self, keyword: str, pre_populate: bool = True) -> List[Tuple[str, str]]:
        if not self.is_loaded:
            self.set_choices(get_fin_periods())

        if pre_populate and keyword.strip() == "":
            return self._choices[: self.size]
        return self.search_keyword(keyword)

    def __repr__(self) -> str:
        return "FINANCIALSPERIOD"



from typing import Any

import click

from i8_terminal.common.cli import log_terminal_usage


@click.group()
def cli() -> None:
    """i8 Terminal - Modern Market Research with the Power of Terminal

    Copyright © 2020-2022 Investoreight | https://www.investoreight.com/"""
    pass


@cli.result_callback()
@click.pass_context
def process_commands(ctx: click.Context, processors: Any) -> None:
    log_terminal_usage(ctx)



from typing import List, Tuple

from i8_terminal.types.auto_complete_choice import AutoCompleteChoice


def get_chart_param_types() -> List[Tuple[str, str]]:
    return [("bar", "Bar chart"), ("line", "Line chart"), ("candlestick", "Candlestick chart")]


class ChartParamType(AutoCompleteChoice):
    name = "charttype"

    def get_suggestions(self, keyword: str, pre_populate: bool = True) -> List[Tuple[str, str]]:
        if not self.is_loaded:
            self.set_choices(get_chart_param_types())

        if pre_populate and keyword.strip() == "":
            return self._choices[: self.size]
        return self.search_keyword(keyword)

    def __repr__(self) -> str:
        return "CHARTTYPE"



from typing import Any, Dict, Optional

import click
from pandas import DataFrame
from rich.console import Console

from i8_terminal.commands.metrics import metrics
from i8_terminal.common.cli import pass_command
from i8_terminal.common.layout import df2Table, format_df
from i8_terminal.common.metrics import get_all_metrics_df


def search_metrics_df(keyword: str) -> Optional[DataFrame]:
    keyword = keyword.lower()
    metrics_df = get_all_metrics_df()
    metrics_df["lower_display_name"] = metrics_df["display_name"].str.lower()
    result = metrics_df[
        metrics_df["metric_name"].str.contains(keyword) | metrics_df["lower_display_name"].str.contains(keyword)
    ]
    if result.empty:
        return None
    return result


def format_metrics_df(df: DataFrame) -> DataFrame:
    formatters: Dict[str, Any] = {}
    col_names = {
        "metric_name": "Metric Name",
        "display_name": "Display Name",
        "unit": "Unit",
        "type": "Type",
        "display_format": "Display Format",
        "data_format": "Data Format",
        "period_type_default": "Default Period Type",
    }
    return format_df(df, col_names, formatters)


@metrics.command()
@click.option("--keyword", "-k", required=True, help="Keyword can be metric name.")
@pass_command
def search(keyword: str) -> None:
    """
    Searches and shows all metrics that match with the given KEYWORD.

    Examples:

    `i8 metrics search --keyword return`

    """
    console = Console()
    with console.status("Fetching data...", spinner="material"):
        df = search_metrics_df(keyword)
    if df is None:
        console.print(f"No metrics found for keyword '{keyword}'", style="yellow")
        return
    df_formatted = format_metrics_df(df)
    table = df2Table(df_formatted)
    console.print(table)



from typing import Any, Dict

import click
import investor8_sdk
from pandas import DataFrame
from rich.console import Console

from i8_terminal.commands.company import company
from i8_terminal.common.cli import pass_command
from i8_terminal.common.layout import df2Table, format_df


def search_stocks_df(keyword: str) -> DataFrame:
    results = investor8_sdk.SearchApi().search_stocks(keyword, 8)
    df = DataFrame([d.to_dict() for d in results])
    return df[["ticker", "name"]]


def format_stocks_df(df: DataFrame, target: str) -> DataFrame:
    formatters: Dict[str, Any] = {}
    col_names = {
        "ticker": "Ticker",
        "name": "Name",
    }
    return format_df(df, col_names, formatters)


@company.command()
@click.option("--keyword", "-k", required=True, help="Keyword can be ticker or company name.")
@pass_command
def search(keyword: str) -> None:
    """
    Searches and shows all securities that match with the given KEYWORD.

    Examples:

    `i8 company search --keyword apl`

    """
    console = Console()
    with console.status("Fetching data...", spinner="material"):
        df = search_stocks_df(keyword)
    df_formatted = format_stocks_df(df, "console")
    table = df2Table(df_formatted)
    console.print(table)
